<?php
/**
 * Module Video
 *
 * @author Friedolin Förder 
 */
namespace modules\piwik;

use \pinion\modules\Module;
use \pinion\modules\FrontendModule;
use \pinion\events\Event;
use \pinion\general\TemplateBuilder;

class Piwik extends Module {
    
    public function install() {
        $this->data
            ->options(null, array(
                "website" => null,
                "directory" => null
            ), true);
    }
    
    public function init() {
        if(!$this->identity) {
            $options = $this->data->options();
            $website = $options["website"] ? $options["website"]."/" : "";
            $directory = $options["directory"];
            if($directory) {
                $url = "$website$directory/";
                $script = <<<SCRIPT
var pkBaseURL = "$url";
document.write(unescape("%3Cscript src='" + pkBaseURL + "piwik.js' type='text/javascript'%3E%3C/script%3E"));
</script><script type="text/javascript">
try {
var piwikTracker = Piwik.getTracker(pkBaseURL + "piwik.php", 1);
piwikTracker.trackPageView();
piwikTracker.enableLinkTracking();
} catch( err ) {}
SCRIPT;
                $this->response->addJsCode($script);
            }
        }
    }
    
    public function addListener() {
        parent::addListener();
        
        if($this->hasPermission("edit options"))       $this->addEventListener("changeOptions");
    }
    
    public function getResources() {
        return array_merge(parent::getResources(), array(
            "edit options"
        ));
    }
    
    public function changeOptions(Event $event) {
        
        $options = array();
        $possibleOptions = array(
            "website",
            "directory"
        );
        
        foreach($possibleOptions as $possibleOption) {
            if($event->hasInfo($possibleOption)) {
                $info = trim($event->getInfo($possibleOption));
                $options[$possibleOption] = $info == "" ? null : $info;
            }
        }
        
        $this->data->options(null, $options);
        
        // add success message
        $this->response->addSuccess($this, $this->translate("The options of the module %s were updated", $this->information["title"]));
    }
    
    public function defineBackend() {
        parent::defineBackend();
        
        $options = $this->data->options();
        
        if($this->hasPermission("edit options")) {
            $this->framework
                ->startGroup("LazyMainTab", array("title" => "Options", "groupEvents" => true))
                    ->input("Textbox", array(
                        "label"   => "directory",
                        "help"    => "The directory where piwik is located",
                        "infoKey" => "directory",
                        "value"   => $options["directory"],
                        "events"  => array("event" => "changeOptions"),
                        "validators" => array("notEmpty" => true)
                    ))
                    ->input("Textbox", array(
                        "label"   => "website",
                        "help"    => "The website where piwik is located. Input is only necessary, if piwik is not located on the current server.",
                        "infoKey" => "website",
                        "value"   => $options["website"],
                        "events"  => array("event" => "changeOptions")
                    ));
        }
        
            
    }
    
        
}


?>