<?php
/**
 * Module Video
 *
 * @author Friedolin Förder 
 */
namespace modules\syntaxhighlighter;

use \pinion\modules\Module;
use \pinion\modules\FrontendModule;
use \pinion\events\Event;
use \pinion\general\TemplateBuilder;

class Syntaxhighlighter extends FrontendModule {
    
    public function install() {
        $this->data
            ->createDataStorage($this->name, array(
                "brush" => array("type" => "text", "translatable" => false),
                "code"  => array("type" => "text", "translatable" => false)
            ))
            ->options(null, array(
                "template" => "Default"
            ), true);
    }
    
    public function getResources() {
        return array_merge(parent::getResources(), array(
            "edit options"
        ));
    }
    
    public function addListener() {
        parent::addListener();
        
        if($this->hasPermission("edit options"))       $this->addEventListener("changeOptions");
    }
    
    public function defineBackend() {
        parent::defineBackend();
        
        $stylesDirs = new \DirectoryIterator(__DIR__."/source/styles");
        $styles = array();
        
        foreach($stylesDirs as $file) {
            $filename = $file->getFilename();
            if(substr($filename, 0, 7) === "shTheme") {
                $styles[] = array("id" => substr($filename, 7, -4));
            }
        }
        
        $this->framework
            ->startGroup("LazyMainTab", array("title" => "Options"))
                ->list("Selector", array(
                    "label"   => "template",
                    "help"    => "What <b>style</b> should be used",
                    "value"   => $this->data->options("template"),
                    "noEmptyValue" => true,
                    "data"    => $styles,
                    "events"  => array("event" => "changeOptions")
                ));
            
    }
    
    public function changeOptions(Event $event) {
        $template = $event->getInfo("template");
        
        $this->data->options("template", $template);
    }
    
    public function setFrontendVars($data) {
        
        $options = $this->data->options();
        
        $this->module("jQuery")->run();
        
        $this->response
            ->addCss("modules/{$this->name}/source/styles/shCore.css")
            ->addCss("modules/{$this->name}/source/styles/shCore{$options["template"]}.css")
            ->addCss("modules/{$this->name}/source/styles/shTheme{$options["template"]}.css")
            ->addJs("modules/{$this->name}/source/scripts/shCore.js")
            ->addJs("modules/{$this->name}/source/scripts/shBrush{$data->brush}.js")
            ->addJsCode("SyntaxHighlighter.all();", "{$this->name}::start");
        
        return $this->preview($data);
    }
    
    public function preview($data) {
        return $data->attributes(array(
            "*",
            "brush" => strtolower($data->brush),
            "code" => htmlentities($data->code)
        ));
    }
    
    public function add(Event $event) {
        
        // the new data
        $data = array();
        
        
        
        // fill the data with information from the event...
        $data["brush"] = $event->getInfo("brush", "Plain");
        $data["code"]  = $event->getInfo("code", "");
        
        
        // create the data storage
        $syntaxhighlighter = $this->data->create($this->name, $data);
        
        
        // add a success message
        $this->response->addSuccess($this, $this->translate("%s added", "<b>".$this->translate($this->information["title"])."</b>"));
        
        // call standard procedure
        $this->_add($event, $syntaxhighlighter);
    }
    
    
    public function edit(Event $event) {
        
        // get the id from the event
        $id = $event->getInfo("id");
        
        
        // get the data storage
        $syntaxhighlighter = $this->data->find($this->name, $id);
        
        
        
        // update the data storage...
        if($event->hasInfo("code")) {
            $syntaxhighlighter->code = $event->getInfo("code");
        }
        if($event->hasInfo("brush")) {
            $syntaxhighlighter->brush = $event->getInfo("brush");
        }
        
        
        // save the data storage
        $syntaxhighlighter->save();
        
        
        // add a success message
        $this->response->addSuccess($this, $this->translate("%s edited", "<b>".$this->translate($this->information["title"])."</b>"));
        
        // call standard procedure
        $this->_edit($event, $syntaxhighlighter);
    }
    
    
    public function defineEditor(Event $event) {
        parent::defineEditor($event);
        
        $settings = $event->getInfo("settings");
        
        $brushesDirs = new \DirectoryIterator(__DIR__."/source/scripts");
        $brushes = array();
        
        foreach($brushesDirs as $file) {
            $filename = $file->getFilename();
            if(substr($filename, 0, 7) === "shBrush") {
                $brushes[] = array("id" => substr($filename, 7, -3));
            }
        }
        
        $this->framework
            ->key("elements")
                ->list("Selector", array(
                    "label" => "language",
                    "infoKey" => "brush",
                    "data" => $brushes,
                    "events" => $settings["events"],
                    "value" => $settings["_isNew"] ? "" : $settings["data"]["brush"],
                    "noEmptyValue" => !$settings["_isNew"]
                ))
                ->input("Codearea", array(
                    "label" => "code",
                    "value" => $settings["_isNew"] ? "" : $settings["data"]["code"],
                    "events" => $settings["events"] 
                ));
    }
        
}


?>