<?php
/**
 * Module Video
 *
 * @author Friedolin Förder 
 */
namespace modules\tagresult;

use \pinion\modules\Module;
use \pinion\modules\FrontendModule;
use \pinion\events\Event;
use \pinion\general\TemplateBuilder;

class Tagresult extends FrontendModule {
    
    public function install() {
        $this->data
            ->createDataStorage($this->name, array(
                "useurl" => array("type" => "boolean"),
                "tag" => array("type" => "varchar", "length" => 100, "isNull" => true)
            ));
    }
    
    public function addListener() {
        parent::addListener();
        
        if($this->identity) {
            $this->addEventListener("getTags");
        }
    }
    
    public function getTags(Event $event) {
        $dbTags = $this->module("tags")->getInformationAboutTags(10000);
        $tags = array();
        foreach($dbTags["tags"] as $dbTag) {
            $tags[] = $dbTag["tag"];
        }
        
        $this->response->setInfo("data", $tags);
    }
    
    public function defineBackend() {
        parent::defineBackend();
        
        // add backend elements with $this->framework...
            
    }
    
        
    public function add(Event $event) {
        
        // the new data
        $data = array();
        
        
        
        // fill the data with information from the event...
        $data["useurl"] = $event->hasInfo("useurl");
        if($event->hasInfo("tag")) {
            $data["tag"] = $event->getInfo("tag");
        }
        
        
        
        // create the data storage
        $tagresult = $this->data->create($this->name, $data);
        
        
        // add a success message
        $this->response->addSuccess($this, $this->translate("%s added", "<b>".$this->translate($this->information["title"])."</b>"));
        
        // call standard procedure
        $this->_add($event, $tagresult);
    }
    
    
    public function edit(Event $event) {
        
        // get the id from the event
        $id = $event->getInfo("id");
        
        
        // get the data storage
        $tagresult = $this->data->find($this->name, $id);
        
        
        
        // update the data storage...
        if($event->hasInfo("useurl")) {
            $tagresult->useurl = $event->getInfo("useurl");
        }
        if($event->hasInfo("tag")) {
            $tagresult->tag = $event->getInfo("tag");
        }
        
        
        // save the data storage
        $tagresult->save();
        
        
        // add a success message
        $this->response->addSuccess($this, $this->translate("%s edited", "<b>".$this->translate($this->information["title"])."</b>"));
        
        // call standard procedure
        $this->_edit($event, $tagresult);
    }
    
    public function defineEditor(Event $event) {
        parent::defineEditor($event);
        
        $settings = $event->getInfo("settings");
        
        $this->framework
            ->key("elements")
                ->input("Checkbox", array(
                    "value" => $settings["_isNew"] ? false : $settings["data"]["useurl"],
                    "label" => "use url",
                    "events" => $settings["events"],
                    "infoKey" => "useurl"
                ))
                ->input("AutoCompleteTextbox", array(
                    "label" => "tag",
                    "value" => $settings["_isNew"] || is_null($settings["data"]["tag"]) ? "" : $settings["data"]["tag"],
                    "events" => $settings["events"],
                    "data" => array(
                        "event" => "getTags"
                    )
                ));
    }   
    
    public function setFrontendVars($data) {
        if($data->useurl) {
            $url = $this->request->getGetParameter("page");
            $url = explode("/", $url);
            array_shift($url);
            $tag = implode("/", $url);
        } else {
            $tag = $data->tag;
        }
                
        return $data->attributes(array(
            "*",
            "tag" => $tag,
            "contents" => $this->module("tags")->getContentsForTag($tag)
        ));
    }
    
    /**
     * Template Render Function: Prints out the html of the content
     * 
     * @param Page_content $element The content of the element
     * 
     * @return null 
     */
    public function content(TemplateBuilder $templateBuilder, $element) {
        $module = $this->module($element->module);
        if($module) {
            $content = new \stdClass();
            $content->templatepath = "tagresult";
            $content->moduleid = $element->moduleid;
            $content->id = -1;
            print $templateBuilder->renderTemplateOfRenderer("_main", $module, $content);
        }
    }
}


?>