<?php
/**
 * Module Video
 *
 * @author Friedolin Förder 
 */
namespace modules\tags;

use \pinion\modules\Module;
use \pinion\modules\FrontendModule;
use \pinion\events\Event;
use \pinion\general\TemplateBuilder;

class Tags extends Module {
    
    public function install() {
        $this->data
            ->createDataStorage("tags", array(
                "module" => array("type" => "varchar", "length" => 100, "translatable" => false),
                "moduleid" => array("type" => "int"),
                "tag" => array("type" => "varchar", "length" => 100, "translatable" => false)
            ), array(
                "revisions" => false
            ));
    }
    
    public function defineBackend() {
        parent::defineBackend();
        
        // add backend elements with $this->framework...
            
    }
    
    public function addListener() {
        parent::addListener();
        
        $this->response->addEventListener("page", "page", $this);
    }
    
    public function page(Event $event) {
        $url = $event->getInfo("url");
        if(substr($url, 0, 4) === "tag/") {
            return "tag";
        }
    }
    
    public function init() {
        $this->addExtension("defineEditor", "tagsEditor");
        $this->addExtension("add", "addTags");
        $this->addExtension("edit", "editTags");
    }
    
    public function tagsEditor(Event $event, Module $module) {
        
        $settings = $event->getInfo("settings");
        
        $value = "";
        if(!$settings["_isNew"]) {
            $dbTags = $this->data->find_all_by_module_and_moduleid("tags", $module->name, $settings["data"]["id"]);
            if(!empty($dbTags)) {
                $tags = array();
                foreach($dbTags as $dbTag) {
                    $tags[] = $dbTag->tag;
                }
                $value = implode(", ", $tags);
            }
        }
        
        $this->framework
            ->key("elements")
            ->input("Textbox", array(
                "label" => "tags",
                "events" => $settings["events"],
                "value" => $value
            ));
    }
    
    public function addTags(Event $event, $moduleContent, Module $module) {
        if($event->hasInfo("tags")) {
            // create new tags
            $tags = explode(", ", $event->getInfo("tags"));
            foreach($tags as $tag) {
                $tag = trim($tag);
                if($tag != "") {
                    $this->data->create("tags", array(
                        "module" => $module->name,
                        "moduleid" => $moduleContent->id,
                        "tag" => $tag
                    ));
                }
            }
        }
    }
    
    public function editTags(Event $event, $moduleContent, Module $module) {
        if($event->hasInfo("tags")) {
            $tags = explode(", ", $event->getInfo("tags"));
            // delete old tags
            $dbTags = $this->data->find_all_by_module_and_moduleid("tags", $module->name, $moduleContent->id);
            foreach($dbTags as $dbTag) {
                $i = -1;
                foreach($tags as $index => $tag) {
                    if($tag === $dbTag->tag) {
                        $i = $index;
                        break;
                    }
                }
                if($i != -1) {
                    array_splice($tags, $i, 1);
                } else {
                    $dbTag->delete();
                }
            }
            
            // create new tags
            foreach($tags as $tag) {
                $tag = trim($tag);
                if($tag != "") {
                    $this->data->create("tags", array(
                        "module" => $module->name,
                        "moduleid" => $moduleContent->id,
                        "tag" => $tag
                    ));
                }
            }
        }
    }
    
    public function tagsByModuleAndId($module, $id) {
        $dbTags = $this->data->find_all_by_module_and_moduleid("tags", $module, $id);
        $tags = array();
        foreach($dbTags as $dbTag) {
            $tags[] = $dbTag->tag;
        }
        return $tags;
    }
    
    public function tags() {
        $renderStackVars = TemplateBuilder::getInstance()->getLastOfRenderStack();
        if(isset($renderStackVars["renderModule"])) {
            $module = $renderStackVars["renderModule"]->name;
        } else {
            return array();
        }
        if(isset($renderStackVars["renderId"])) {
            $id = $renderStackVars["renderId"];
        } else {
            return array();
        }
        return $this->tagsByModuleAndId($module, $id);
    }
    
    public function getInformationAboutTags($limit = 50) {
        $dbTags = $this->data->all("tags", array(
            "select" => "tag, count(tag) as count, created",
            "group" => "tag",
            "limit" => $limit,
            "order" => "count DESC"
        ));
        
        $tags = $this->data->getAttributes($dbTags);
        if(empty($tags)) {
            return array("min" => 0, "max" => 0, "num" => 0, "tags" => $tags);
        } else {
            $num = count($tags);
            $max = $tags[0]["count"];
            $min = $tags[$num-1]["count"];
        }
        uasort($tags, function($a, $b) {
            $a = $a["tag"];
            $b = $b["tag"];
            
            if ($a == $b) {
                return 0;
            }
            return ($a < $b) ? -1 : 1;
        });
        
        return array("min" => $min, "max" => $max, "num" => $num, "tags" => $tags);
    }
    
    /**
     * 
     * 
     * @param string $str The tag
     */
    public function getContentsForTag($str) {
        $dbTags = $this->data->find_all_by_tag("tags", $str);
        $contents = array();
        foreach($dbTags as $dbTag) {
            $content = $this->module("page")->data->find_by_module_and_moduleid("content", $dbTag->module, $dbTag->moduleid);
            if(is_object($content)) {
                $contents[] = $content;
            } else {
                $dbTag->delete();
            }
        }
        return $contents;
    }
}


?>